/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// grid.h                                                                  //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// Grid control library - Version 1.20 - 1 nov 1994                        //
// Original code: H. Picot                                                 //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

#ifndef GRID_H
#define GRID_H

#define HREZSIZE 500

#ifndef __OWL_SCROLLBAR_H
#include "owl\scrollba.h"
#endif


class _GRIDCLASS TGDialog : public TDialog
   {
   protected:
      TColor bkg;
      HBRUSH brush;
      virtual HBRUSH EvCtlColor(HDC hDC, HWND hWndChild, UINT ctlType);
      virtual HBRUSH CtlColors (HDC hDC);

   public:
      TGDialog (TWindow *p, TResId resId,
               TColor bkg = GetSysColor (COLOR_WINDOW), TModule *module = 0);
     ~TGDialog (); 
      virtual void   SetColor  (TColor bkg);
      virtual TColor BkgColor  () { return bkg; }
      virtual HBRUSH BkgBrush  () { return brush; }

   DECLARE_RESPONSE_TABLE (TGDialog);
   };


#define CODE_COLORS(type)                                                  \
HBRUSH type::CtlColors (HDC hDC)                                           \
   {                                                                       \
   TDC dc(hDC);                                                            \
   dc.SetTextColor (text);                                                 \
   dc.SetBkColor (bkg);                                                    \
   return brush;                                                           \
   }                                                                       \
void type::SetColors (TColor t, TColor b)                                  \
   {                                                                       \
   text = t;                                                               \
   bkg = b;                                                                \
   DeleteObject (brush);                                                   \
   brush = CreateSolidBrush (COLORREF(bkg));                               \
   }

#define HEADER_COLORS                                                      \
   protected:                                                              \
      TColor text;                                                         \
      TColor bkg;                                                          \
      HBRUSH brush;                                                        \
   public:                                                                 \
      virtual HBRUSH CtlColors (HDC hDC);                                  \
      virtual void   SetColors (TColor text,                               \
                                TColor bkg = GetSysColor(COLOR_WINDOW));   \
      virtual TColor BkgColor ()  { return bkg; }                          \
      virtual TColor TextColor () { return text; }                         \
      virtual HBRUSH BkgBrush  () { return brush; }

class _GRIDCLASS TGCheckBox;
class _GRIDCLASS TGComboBox;
class _GRIDCLASS TGEdit;
class _GRIDCLASS TGStatic;
class _GRIDCLASS TGButton;

void _GRIDFUNCTION Frame3d (TDC &dc, TRect &rc, int w,
                            TColor topleft, TColor bottomright); // new in 1.20
void _GRIDFUNCTION Frame3d (TDC &dc, int x0, int y0, int x1, int y1, int w,
                            TColor topleft, TColor bottomright); // changed in 1.20


class _GRIDCLASS TGrid : public TControl
   {
   public:
      typedef enum Align  { Left=DT_LEFT, Center=DT_CENTER, Right=DT_RIGHT };
      typedef enum Type       { gEdit, gStatic, gCombo, gButton, gCheck };
      typedef enum EnterKey   { EnterNextColumn, NextLine, EndDialog, DoNothing }; // changed in 1.20
      typedef enum TabKey     { TabNextColumn, AutoNextLine, NextControl }; // new in 1.20
      typedef enum Attributes { gNoAttrib = 0, gVScroll = 1, gHScroll = 2,
                                gTitle = 4, gVBar = 8, gAllAttrib = 15 }; // changed in 1.20
      typedef enum LineMarker { ArrowOnly, ArrowAndText, ReverseColor }; // new in 1.20

   protected:
      class _GRIDCLASS Frame : public TControl
         {
         protected:
            TGrid *grid;

            void DrawCorner (TDC &dc, TRect &rc);
            BOOL EvEraseBkgnd (HDC) { return TRUE; }
            void Paint (TDC &dc, BOOL erase, TRect &clip);

         public:
            Frame (TGrid *g);
            DECLARE_RESPONSE_TABLE (TGrid::Frame);
         };

      class _GRIDCLASS Element
         {
         public:
            TGrid *grid;
            TControl *control;
            Element *next;
            Element *previous;
            int left;
            int width;
            TGrid::Align align;
            TGrid::Type  type;
            string title;                                      // new in 1.20

            Element (TControl *c, Element *prev, TGrid *grid, const char *t); // changed in 1.20
           ~Element ();
            int CtrlWidth () const;                        // changed in 1.20
            int CtrlHeight () const;                       // changed in 1.20
            void GetData (char * &data, int l) const;          // new in 1.20
            int  GetDataLength () const;                       // new in 1.20
            BOOL SetFocus ();
            void SetData (const char *data);                   // new in 1.20
            void SetTitle (const char *newtitle);              // new in 1.20
         };

      class _GRIDCLASS VScroll : public TScrollBar
         {
         public:
            TGrid *grid;

            VScroll (TGrid *g);

            void SBBottom ();
            void SBLineDown ();
            void SBLineUp ();
            void SBPageDown ();
            void SBPageUp ();
            void SBThumbPosition (int pos);
            void SBThumbTrack (int pos);
            void SBTop ();
         };

      class _GRIDCLASS HScroll : public TScrollBar
         {
         public:
            TGrid *grid;

            HScroll (TGrid *g);

            void SBBottom ();
            void SBLineDown ();
            void SBLineUp ();
            void SBPageDown ();
            void SBPageUp ();
            void SBThumbPosition (int pos);
            void SBThumbTrack (int pos);
            void SBTop ();
         };

      class _GRIDCLASS Title : public TControl
         {
         friend TGrid;

         protected:
            TGrid *grid;
            // char * text;                                // removed in 1.20
            void DrawCell (TDC &dc, int x, int w, const char *p, 
                                                             TGrid::Align a);
            int CalcHeight ();                                 // new in 1.20
            void Paint (TDC &dc, BOOL erase, TRect &);
            void SetupWindow ();

         public:
            Title (TGrid *g);                              // changed in 1.20
         };

      class _GRIDCLASS VerticalBar : public TControl
         {
         friend TGrid;

         protected:
            TGrid *grid;
            BOOL lineNumber;
            int charWidth;

            int CalcWidth ();                                  // new in 1.20
            void DrawCell (TDC &dc, TRect &rc, int line);
            void EvLButtonDown (UINT modKeys, TPoint &point);  // new in 1.20
            TRect FillRect (TDC &dc, int line, BOOL erase) const;// changed in 1.20
            char *Info (int line) const;                   // changed in 1.20
            void Paint (TDC &dc, BOOL erase, TRect &);
            void SetupWindow ();
            void ShowMarker (int line) const;              // changed in 1.20
            void ShowText (int line) const;                // changed in 1.20

         public:
            VerticalBar (TGrid *g);                        // changed in 1.20
            void SetUserText (int cwidth);                     // new in 1.20

            DECLARE_RESPONSE_TABLE (TGrid::VerticalBar);
         };

      friend Frame;
      friend Element;
      friend VScroll;
      friend HScroll;
      friend Title;
      friend VerticalBar;
      friend TGCheckBox;
      friend TGComboBox;
      friend TGEdit;
      friend TGStatic;
      friend TGButton;
      Title   *title;
      Element *firstControl;
      Element *lastControl;
      Element *current;
      BOOL firstPaint;
      BOOL lineSwapping;
      int lineHeight;
      int firstLine;    // offset of first visible line
      int activeLine;   // offset of visible active line
      int visibleLines; // number of visible lines
      int maxLines;     // maximum number of lines (-1 = no limite)
      int nLines;       // actual number of lines, if maxLines == -1
      int nControls;    // number of columns with a control
      int hpos;         // current horizontal offset (0 <= hpos < totalWidth)
      int totalWidth;   // total width for all columns; must be >= Attr.W  
      VScroll *vscroll;
      HScroll *hscroll;
      VerticalBar *vbar;
      Frame *frame;
      TColor lineColor;
      EnterKey enterKey;
      TabKey   tabKey;                                         // new in 1.20
      LineMarker lineMarker;                                   // new in 1.20
      TFont *gridFont;                                     // changed in 1.20
      TFont *titleFont;                                        // new in 1.20
      TBitmap *ComboBmp;                                       // new in 1.20
      TBitmap *CheckBoxesBmp;                                  // new in 1.20
      TBitmap *BwccCheckBmp;                                   // new in 1.20
      TBitmap *BwccUncheckBmp;                                 // new in 1.20
      LOGFONT *gridLogFont;                                    // new in 1.20
      LOGFONT *titleLogFont;                                   // new in 1.20

      // virtual protected methods ------------------------------------------
      virtual BOOL AppendNewLine (int &newline) { return TRUE; }// new in 1.20
      virtual void CalcPos ();
      virtual void ChangeWindowPos (int &x, int &y, int &w, int &h,
                                    UINT flags);
      virtual void GotoFirstColumn ();
      virtual void GotoLastColumn ();
      virtual BOOL GotoNextColumn ();
      virtual BOOL GotoPreviousColumn ();
      virtual void Init (int max, EnterKey ek, TabKey tk, LineMarker lm,
                         int attrib);                      // changed in 1.20
      virtual BOOL KeyDown (UINT key, int repeat);
      virtual void MoveHPos (int delta);
      virtual void ProcessMouse (UINT modKeys, TPoint &point, UINT message);
      virtual void SetFocusOnChild (TControl *child);
      virtual void SetFont ();                                 // new in 1.20
      virtual void SetFont (char *facename, int height = 0,
                            int weight = FW_BOLD, BOOL italic = FALSE);// new in 1.20
      virtual void SetHPos (int newpos);
      virtual void SetTitleFont ();                            // new in 1.20
      virtual void SetTitleFont (char *facename, int height =0,
                                 int weight =FW_BOLD, BOOL italic =FALSE);// new in 1.20
      virtual void SetVBarText (int cwidth = 4);               // new in 1.20
      virtual void TransferLine (int line, TTransferDirection direction);

      // message processing and TControl's overloaded methods ---------------
      virtual BOOL CanClose ();
      virtual HBRUSH EvCtlColor(HDC hDC, HWND hWndChild, UINT ctlType);
      virtual BOOL EvEraseBkgnd (HDC) { return TRUE; }
      virtual void EvLButtonDown (UINT modKeys, TPoint &point);
      virtual void EvLButtonDblClk (UINT modKeys, TPoint &point);
      virtual void EvPaint ();
      virtual BOOL EvSetCursor (HWND hwnd, UINT hitTest, UINT mouseMsg);
      virtual void EvSetFocus (HWND);
      virtual void EvWindowPosChanging (WINDOWPOS far &wpos);
      virtual void Paint (TDC &dc, BOOL erase, TRect &rc);
      // virtual void SetupWindow ();                      // removed in 1.20

      // keyboard handling --------------------------------------------------
      virtual BOOL VkDelete (BOOL ctrl, BOOL shift, int repeat);
      virtual BOOL VkDown   (BOOL ctrl, BOOL shift, int repeat);
      virtual BOOL VkEnd    (BOOL ctrl, BOOL shift, int repeat);
      virtual BOOL VkEscape (BOOL ctrl, BOOL shift, int repeat);
      virtual BOOL VkHome   (BOOL ctrl, BOOL shift, int repeat);
      virtual BOOL VkInsert (BOOL ctrl, BOOL shift, int repeat);
      virtual BOOL VkLeft   (BOOL ctrl, BOOL shift, int repeat);
      virtual BOOL VkNext   (BOOL ctrl, BOOL shift, int repeat);
      virtual BOOL VkPrior  (BOOL ctrl, BOOL shift, int repeat);
      virtual BOOL VkReturn (BOOL ctrl, BOOL shift, int repeat);
      virtual BOOL VkRight  (BOOL ctrl, BOOL shift, int repeat);
      virtual BOOL VkTab    (BOOL ctrl, BOOL shift, int repeat);
      virtual BOOL VkUp     (BOOL ctrl, BOOL shift, int repeat);

   public:
      // constructors and destructor ----------------------------------------
      TGrid (TWindow *p, int id, int x, int y, int w, int h,
             EnterKey ek = NextLine, TabKey = TabNextColumn,
             LineMarker lm = ReverseColor, int max = -1,
             int attrib = gAllAttrib);                     // changed in 1.20
      TGrid (TWindow *p, int id, EnterKey ek = NextLine,
             TabKey = TabNextColumn, LineMarker lm = ReverseColor,
             int max = -1, int attrib = gAllAttrib);       // changed in 1.20
     ~TGrid ();

      // public methods -----------------------------------------------------
      Element *FindElement (HWND hwnd)   const;
      Element *FindElement (int id)      const;
      Element *FindElement (TControl *c) const;
      Element *FindElement (TPoint &p)   const;
      int GetActiveLine ()    const { return activeLine; }
      int GetFirstLine ()     const { return firstLine;  }
      int GetCurrentLine ()   const { return firstLine+activeLine; }
      BOOL GetLineSwapping () const { return lineSwapping; }
      void SetEnterKey (EnterKey ek) { enterKey = ek; }        // new in 1.20
      void SetLineColor (TColor c) { lineColor = c; }
      void SetLineMarker (LineMarker lm);                      // new in 1.20
      void SetLineSwapping (BOOL t) { lineSwapping = t; }
      void SetTabKey (TabKey tk) { tabKey = tk; }              // new in 1.20

      // virtual public methods ---------------------------------------------
      virtual void Add (TControl *p, const char *title, Align a = Left); // changed in 1.20
      // virtual void AddVBar (BOOL linenumber = TRUE, int cwidth = 0);  // removed in 1.20
      virtual void ChangeTitle (int id, const char *newTitle); // new in 1.20
      virtual void Go (int pos);
      virtual const char *GetTitle (int id) const;             // new in 1.20
      virtual void Move (int delta);
      virtual void Refresh ();
      virtual void RefreshItem (int line, int id);
      virtual void RefreshLine (int line);
      virtual void ResetPosition ();
      virtual void SetLineHeight (int h);
      virtual void SetNLines (int n);
      virtual void SwapLines (int l1, int l2);
      virtual BOOL TransferActiveLine (TTransferDirection td);

      // virtual public methods which will usually be overloaded ------------
      virtual BOOL DeleteLine (int line) { return FALSE; }
      virtual BOOL InsertLine (int line) { return FALSE; }
      virtual BOOL IsValidLine (int line) { return TRUE; }
      virtual char *TransferItem (int line, int id) { return 0; }
      virtual void TransferItem (int line, int id, const char *data) {}

   DECLARE_RESPONSE_TABLE (TGrid);
   };

#define GRID_EV_GETDLGCODE(derived,base)                                    \
   UINT derived::EvGetDlgCode (MSG far *msg)                                \
      {                                                                     \
      if (grid)                                                             \
         return base::EvGetDlgCode (msg) | DLGC_WANTALLKEYS;                \
      else                                                                  \
         return base::EvGetDlgCode (msg);                                   \
      }

#define GRID_EV_KEYUP(derived,base)                                         \
   void derived::EvKeyUp (UINT key, UINT repeat, UINT flags)                \
      {                                                                     \
      if (grid)                                                             \
         {                                                                  \
         switch (key)                                                       \
            {                                                               \
            case VK_INSERT:                                                 \
            case VK_DELETE:                                                 \
            case VK_HOME:                                                   \
            case VK_END:                                                    \
            case VK_ESCAPE:                                                 \
            case VK_TAB:                                                    \
            case VK_RETURN:                                                 \
            case VK_UP:                                                     \
            case VK_DOWN:                                                   \
            case VK_PRIOR:                                                  \
            case VK_NEXT:                                                   \
               return;                                                      \
            }                                                               \
         }                                                                  \
      base::EvKeyUp (key, repeat, flags);                                   \
      }


#define GRID_EV_KEYDOWN(derived,base)                                       \
   void derived::EvKeyDown (UINT key, UINT repeat, UINT flags)              \
      {                                                                     \
      if (grid)                                                             \
         {                                                                  \
         switch (key)                                                       \
            {                                                               \
            case VK_HOME:                                                   \
            case VK_END:                                                    \
            case VK_TAB:                                                    \
            case VK_RETURN:                                                 \
            case VK_UP:                                                     \
            case VK_DOWN:                                                   \
            case VK_PRIOR:                                                  \
            case VK_NEXT:                                                   \
            case VK_INSERT:                                                 \
            case VK_DELETE:                                                 \
            case VK_ESCAPE:                                                 \
               if (grid->KeyDown (key, repeat))                             \
                  return;                                                   \
               break;                                                       \
            }                                                               \
         }                                                                  \
      base::EvKeyDown (key, repeat, flags);                                 \
      }


#define GRID_EV_SETFOCUS(derived,base)                                      \
   void derived::EvSetFocus (HWND hWndLostFocus)                            \
      {                                                                     \
      if (grid)                                                             \
         grid->SetFocusOnChild (this);                                      \
      base::EvSetFocus (hWndLostFocus);                                     \
      }


#define GRID_EV_KEYUP_BUTTON(derived,base)                                  \
   void derived::EvKeyUp (UINT key, UINT repeat, UINT flags)                \
      {                                                                     \
      if (grid)                                                             \
         {                                                                  \
         switch (key)                                                       \
            {                                                               \
            case VK_INSERT:                                                 \
            case VK_DELETE:                                                 \
            case VK_HOME:                                                   \
            case VK_END:                                                    \
            case VK_ESCAPE:                                                 \
            case VK_TAB:                                                    \
            case VK_RETURN:                                                 \
            case VK_UP:                                                     \
            case VK_DOWN:                                                   \
            case VK_PRIOR:                                                  \
            case VK_NEXT:                                                   \
            case VK_LEFT:                                                   \
            case VK_RIGHT:                                                  \
               return;                                                      \
            }                                                               \
         }                                                                  \
      base::EvKeyUp (key, repeat, flags);                                   \
      }


#define GRID_EV_KEYDOWN_BUTTON(derived,base)                                \
   void derived::EvKeyDown (UINT key, UINT repeat, UINT flags)              \
      {                                                                     \
      if (grid)                                                             \
         {                                                                  \
         switch (key)                                                       \
            {                                                               \
            case VK_HOME:                                                   \
            case VK_END:                                                    \
            case VK_ESCAPE:                                                 \
            case VK_INSERT:                                                 \
            case VK_DELETE:                                                 \
            case VK_TAB:                                                    \
            case VK_RETURN:                                                 \
            case VK_UP:                                                     \
            case VK_DOWN:                                                   \
            case VK_PRIOR:                                                  \
            case VK_NEXT:                                                   \
            case VK_LEFT:                                                   \
            case VK_RIGHT:                                                  \
               if (grid->KeyDown (key, repeat))                             \
                  return;                                                   \
               break;                                                       \
            }                                                               \
         }                                                                  \
      base::EvKeyDown (key, repeat, flags);                                 \
      }


#endif

// EOF
