/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// gridcbox.cpp                                                            //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// Grid control library - Version 1.20 - 1 nov 1994                        //
// Original code: H. Picot                                                 //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

#include "griddll.h"

DEFINE_RESPONSE_TABLE1 (TGComboBox, TComboBox)
   EV_WM_GETDLGCODE,
   EV_WM_KEYUP,
   EV_WM_KEYDOWN,
   EV_WM_SETFOCUS,
   EV_WM_CTLCOLOR,
END_RESPONSE_TABLE;


TGComboBox::TGComboBox (TWindow *p, int resid, UINT textlen, TModule *module)
          : TComboBox (p, resid, textlen, module)
   {
   grid = 0;
   hEdit = 0;
   bkg         = GetSysColor (COLOR_WINDOW);
   text        = GetSysColor (COLOR_WINDOWTEXT);
   brush       = CreateSolidBrush (bkg);
   activeBG    = GetSysColor (COLOR_HIGHLIGHT);
   selectedFG  = GetSysColor (COLOR_HIGHLIGHTTEXT);
   }


TGComboBox::TGComboBox (TWindow *p, int resid, int x, int y, int w, int h,
                        DWORD style, UINT textlen, TModule *module)
          : TComboBox (p, resid, x, y, w, h, style, textlen, module)
   {
   grid = 0;
   hEdit = 0;
   bkg         = GetSysColor (COLOR_WINDOW);
   text        = GetSysColor (COLOR_WINDOWTEXT);
   brush       = CreateSolidBrush (bkg);
   activeBG    = GetSysColor (COLOR_HIGHLIGHT);
   selectedFG  = GetSysColor (COLOR_HIGHLIGHTTEXT);
   }


TGComboBox::~TGComboBox ()
   {
   DeleteObject (brush);
   }


void TGComboBox::SetupWindow ()
   {
   TComboBox::SetupWindow ();
   hEdit = GetDlgItem (1001);
   }


GRID_EV_SETFOCUS(TGComboBox, TComboBox)
GRID_EV_GETDLGCODE(TGComboBox,TComboBox)
CODE_COLORS(TGComboBox)


// ----------------------------------------------------------------------- //
// this method supports custom colors for the edit portion of the combobox //
// (for editable combo boxes)                                              //
// ----------------------------------------------------------------------- //
HBRUSH TGComboBox::EvCtlColor(HDC hDC, HWND hWndChild, UINT)
   {
   if (IsChild (hWndChild))
      return CtlColors (hDC);
   return 0;
   }


BOOL TGComboBox::PreProcessMsg (MSG &msg)
   {
   if (grid && msg.message == WM_KEYDOWN)
      {
      switch (msg.wParam)
         {
         case VK_NEXT:
         case VK_PRIOR:
         case VK_UP:
         case VK_DOWN:
         case VK_HOME:
         case VK_END:
            if (GetDroppedState ())
               break;
         case VK_ESCAPE:
         case VK_INSERT:
         case VK_DELETE:
         case VK_TAB:
         case VK_RETURN:
            if (grid->KeyDown (msg.wParam, LOWORD(msg.lParam)))
               return TRUE;
            break;
         }
      }

   return TComboBox::PreProcessMsg (msg);
   }


void TGComboBox::EvKeyUp (UINT key, UINT repeat, UINT flags)
   {
   if (grid)
      {
      switch (key)
         {
         case VK_NEXT:
         case VK_PRIOR:
         case VK_UP:
         case VK_DOWN:
         case VK_HOME:
         case VK_END:
            if (GetDroppedState ())
               break;
         case VK_ESCAPE:
         case VK_INSERT:
         case VK_DELETE:
         case VK_TAB:
         case VK_RETURN:
            return;
         }
      }
   TComboBox::EvKeyUp (key, repeat, flags);
   }


void TGComboBox::EvKeyDown (UINT key, UINT repeat, UINT flags)
   {
   if (grid)
      {
      switch (key)
         {
         case VK_NEXT:
         case VK_PRIOR:
         case VK_UP:
         case VK_DOWN:
         case VK_HOME:
         case VK_END:
            if (GetDroppedState ())
               break;
         case VK_ESCAPE:
         case VK_INSERT:
         case VK_DELETE:
         case VK_TAB:
         case VK_RETURN:
            if (grid->KeyDown (key, repeat))
               return;
            break;
         }
      }
   TComboBox::EvKeyDown (key, repeat, flags);
   }


void TGComboBox::MeasureItem (MEASUREITEMSTRUCT &info)
   {
   info.itemHeight = HIWORD(GetDialogBaseUnits ());
   }


void TGComboBox::ODASelect (DRAWITEMSTRUCT &info)
   {
   DrawCBItem (info);
   }


void TGComboBox::ODADrawEntire (DRAWITEMSTRUCT &info)
   {
   DrawCBItem (info);
   }


void TGComboBox::DrawCBItem (DRAWITEMSTRUCT &info)
   {
   TDC dc (info.hDC);
   char *p;
   TRect rect (info.rcItem);

   InitDraw (info, dc, p, rect);
   if (p)
      {
      dc.DrawText (p, -1, rect, (WORD)(DT_LEFT|DT_VCENTER|DT_SINGLELINE));
      delete [] p;
      }
   }


void TGComboBox::InitDraw (DRAWITEMSTRUCT &info, TDC &dc, char * &p, TRect &rect)
   {
   int n = info.itemID;
   UINT &state = info.itemState;
   TColor b = (state&ODS_SELECTED ? activeBG : bkg);
   dc.FillRect (rect, TBrush(b));
   dc.SetBkColor (b);
   dc.SetTextColor (state&ODS_DISABLED ?
                    text : (state&ODS_SELECTED ? selectedFG : text));

   int l = (n >= 0 ? GetStringLen (n) : 0);
   p = (l ? new char[l+1] : 0);
   if (p)
      GetString (p, n);
   }


int TGComboBox::Height ()
   {
   // Empirical methods for determining height. There seems to be no logic
   // in the way the height of the edit part is determined by Windows!
   BOOL ownerdraw = (Attr.Style&(CBS_OWNERDRAWFIXED|CBS_OWNERDRAWVARIABLE)) ? TRUE : FALSE;

   if (IsWindow() && grid && grid->gridFont && !ownerdraw)
      {
      TClientDC dc(*this);
      dc.SelectObject (*grid->gridFont);
      TEXTMETRIC tm;
      dc.GetTextMetrics (tm);
      dc.RestoreFont ();
      int h = tm.tmHeight;
      if (h < 8)
         return h + 5;
      else if (h < 12)
         return h + 6;
      else if (h < 16)
         return h + 7;
      else if (h < 20)
         return h + 8;
      else
         return h + 9;
      }

   int h = HIWORD(GetDialogBaseUnits());
   int d = h > 16 ? -1 : 0;
   if (ownerdraw)
      return (h*3)/2 + d;
   else
      return (h*5)/4 + d;
   }



void TGComboBox::DrawCell (TDC &dc, TRect &rc, const char *p, TGrid::Align)
   {
   // flags
   BOOL Editable = (Attr.Style&CBS_DROPDOWNLIST)==CBS_DROPDOWNLIST ? FALSE : TRUE;

   // colors
   TColor btnFace  = GetSysColor (COLOR_BTNFACE);
   TColor btnText  = GetSysColor (COLOR_BTNTEXT);
   TColor wndFrame = GetSysColor (COLOR_WINDOWFRAME);
   TColor window   = GetSysColor (COLOR_WINDOW);

   // edit background & frame
   int w = LOWORD(GetDialogBaseUnits());

   // Adjust height of rectangle
   int ch = Height ();
   if (rc.Height () > ch)
      {
      TRect r (rc);
      r.top += ch;
      dc.FillRect (r, brush);
      rc.bottom = r.top;
      }

   // split rc into rc (left) and rc1 (right)
   TRect rc1 (rc);
   rc1.left = rc1.right-w-w-1;
   rc.right = rc1.left+1;
   if (Editable)
      {
      TRect rc2 (rc);
      rc2.left = rc2.right-w;
      dc.FillRect (rc2, brush);
      rc.right = rc2.left;
      }

   dc.SelectObject (TPen (wndFrame));

   dc.SelectObject (brush);
   dc.Rectangle (rc);
   dc.RestoreBrush ();

   // button rectangle
   dc.SelectObject (TBrush (btnFace));
   dc.Rectangle (rc1);
   dc.RestoreBrush ();
   dc.RestorePen ();

   // drop down button frame
   int x0 = rc1.left+1;
   int y0 = rc1.top+1;
   int x1 = rc1.right-2;
   int y1 = rc1.bottom-2;
   TColor shadow = GetSysColor (COLOR_BTNSHADOW);
   Frame3d (dc, x0, y0, x1, y1, 1, GetSysColor (COLOR_BTNHIGHLIGHT), shadow);
   x0++, y0++, x1--, y1--;
   dc.SelectObject (TPen (shadow));
   dc.MoveTo (x0, y1);
   dc.LineTo (x1, y1);
   dc.LineTo (x1, y0);
   dc.RestorePen ();

   // draw bitmap
   dc.SetBkColor (btnFace);
   dc.SetTextColor (btnText);
   TMemoryDC memDC (dc);
   if (grid->ComboBmp == 0)
      grid->ComboBmp = new TBitmap (HINSTANCE(0), OBM_COMBO);
   int bmpw = grid->ComboBmp->Width ();
   int bmph = grid->ComboBmp->Height ();
   int x = rc1.left+(rc1.Width()-bmpw)/2;
   int y = rc1.top+(Height()-bmph)/2;
   memDC.SelectObject (*grid->ComboBmp);
   rc1.Set (x, y, x+bmpw, y+bmph);
   dc.BitBlt (rc1, memDC, TPoint(0,0));
   memDC.RestoreBitmap ();

   // text
   if (p)
      {
      rc.Inflate (-3,-2);
      WORD align = (WORD)(DT_LEFT|DT_VCENTER|DT_SINGLELINE);
      dc.SetBkColor (bkg);
      dc.SetTextColor (text);
      dc.DrawText (p, -1, rc, align);
      }
   }

// EOF
