/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// gridckbx.cpp                                                            //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// Grid control library - Version 1.20 - 1 nov 1994                        //
// Original code: H. Picot                                                 //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

#include "griddll.h"


DEFINE_RESPONSE_TABLE1 (TGCheckBox, TCheckBox)
   EV_WM_GETDLGCODE,
   EV_WM_KEYUP,
   EV_WM_KEYDOWN,
   EV_WM_SETFOCUS,
END_RESPONSE_TABLE;


TGCheckBox::TGCheckBox (TWindow *p, UINT id, TGroupBox *group, TModule *module)
          : TCheckBox (p, id, group, module)
   {
   grid = 0;
   bkg   = GetSysColor (COLOR_WINDOW);
   brush = CreateSolidBrush (bkg);
   text  = GetSysColor (COLOR_WINDOWTEXT);
   }


TGCheckBox::TGCheckBox (TWindow *p, UINT id, char *title, int x, int y, int w, int h, TGroupBox *group, TModule *module)
          : TCheckBox (p, id, title, x, y, w, h, group, module)
   {
   grid = 0;
   bkg   = GetSysColor (COLOR_WINDOW);
   brush = CreateSolidBrush (bkg);
   text  = GetSysColor (COLOR_WINDOWTEXT);
   }


TGCheckBox::~TGCheckBox ()
   {
   DeleteObject (brush);
   }


GRID_EV_SETFOCUS(TGCheckBox, TCheckBox)
GRID_EV_GETDLGCODE(TGCheckBox, TCheckBox)
GRID_EV_KEYDOWN_BUTTON(TGCheckBox,TCheckBox)
GRID_EV_KEYUP_BUTTON(TGCheckBox,TCheckBox)
CODE_COLORS(TGCheckBox)


BOOL TGCheckBox::PreProcessMsg (MSG &msg)
   {
   if (grid &&
       msg.message == WM_KEYDOWN &&
       msg.wParam == VK_ESCAPE   &&
       grid->KeyDown (msg.wParam, LOWORD(msg.lParam)) )
       return TRUE;

   return TCheckBox::PreProcessMsg (msg);
   }


void TGCheckBox::DrawCell (TDC &dc, TRect &rc, const char *p, TGrid::Align align)
   {
   BOOL checked = FALSE;
   if (p && *p > '0')
      checked = TRUE;

   TModule *bwcc = 0;
   TApplication *app = GetApplication ();

   if (app->BWCCEnabled() && strnicmp ("BorCheck", GetClassName(), strlen("BorCheck")) == 0)
      bwcc = app->GetBWCCModule ();

   TRect rc1 (rc);
   if (bwcc == 0)   // standard Windows check and radio boxes
      {
      dc.FillRect (rc, TBrush (bkg));
      if (grid->CheckBoxesBmp == 0)
         grid->CheckBoxesBmp = new TBitmap (HINSTANCE(0), OBM_CHECKBOXES);
      int w = grid->CheckBoxesBmp->Width()/4;
      int h = grid->CheckBoxesBmp->Height()/3;
      if (align == TGrid::Left)
         rc1.right = rc1.left+w;
      else
         rc1.left = rc1.right-w;
      rc1.bottom = rc1.top+h;
      rc1.Offset (0, (rc.Height() - h) / 2);
      TMemoryDC memDC (dc);
      memDC.SelectObject (*grid->CheckBoxesBmp);
      dc.BitBlt (rc1, memDC, TPoint(checked?w:0, 0));
      memDC.RestoreBitmap ();
      if (align == TGrid::Left)
         rc.left = rc1.right+5;
      else
         {
         rc.right = rc1.left-5;
         rc.left++;
         }
      }
   else  // if (bwcc)
      {
      bkg = TColor::LtGray;
      text = TColor::Black;
      dc.FillRect (rc, TBrush (bkg));
      TBitmap *bmp;
      if (checked)
         {
         if (grid->BwccCheckBmp == 0)
            grid->BwccCheckBmp = new TBitmap (bwcc->GetInstance(), 111);
         bmp = grid->BwccCheckBmp;
         }
      else
         {
         if (grid->BwccUncheckBmp == 0)
            grid->BwccUncheckBmp = new TBitmap (bwcc->GetInstance(), 110);
         bmp = grid->BwccUncheckBmp;
         }
      int w = bmp->Width();
      int h = bmp->Height();
      rc1.right = rc1.left+w;
      rc1.bottom = rc1.top+h;
      rc1.Offset (0, (rc.Height() - h) / 2);
      TMemoryDC memDC (dc);
      memDC.SelectObject (*bmp);
      dc.BitBlt (rc1, memDC, TPoint(0, 0));
      memDC.RestoreBitmap ();
      align = TGrid::Left;
      rc.left = rc1.right+6;
      }

   dc.SetBkColor (bkg);
   dc.SetTextColor (text);
   char *txt = 0;
   int len = GetWindowTextLength ()+1;
   if (len > 1)
      {
      txt = new char[len];
      GetWindowText (txt, len);
      dc.DrawText (txt, -1, rc, (WORD)(DT_LEFT|DT_VCENTER|DT_SINGLELINE));
      delete [] txt;
      }
   }

// EOF
