/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// test.cpp                                                                //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// Grid control library - Version 1.20 - 1 nov 1994                        //
// Original code: H. Picot                                                 //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

#include "griddll.h"
#pragma hdrstop
#include <owl\framewin.h>
#include "test.h"
#include "test.rh"
#include "typeinfo.h"

// OwlMain //////////////////////////////////////////////////////////////////
int OwlMain (int, char * [])
   {
   return Application ().Run ();
   }


// class Application ////////////////////////////////////////////////////////
void Application::InitMainWindow ()
   {
   EnableBWCC(TRUE);
   TFrameWindow *frame =
                  new TFrameWindow (0, "TGrid 1.20 demo", new Test(0), TRUE);
   SetMainWindow (frame);
   }


/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// Class Test                                                              //
//                                                                         //
//-------------------------------------------------------------------------//
//                                                                         //
// This class is derived from TGDialog                                     //
// It may be derived from TDialog is color support is not needed           //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////
DEFINE_RESPONSE_TABLE1 (Test, TGDialog)
   EV_COMMAND (107, TestRefresh),
   EV_WM_WINDOWPOSCHANGED,
END_RESPONSE_TABLE;

// constructor //////////////////////////////////////////////////////////////
Test::Test (TWindow *p) : TGDialog (p, DIALOG_TEST)
   {
   // create interface objects, just as if there were no grid.
   ok = new TButton (this, IDOK);
   esc = new TButton (this, IDCANCEL);
   test = new TButton (this, 107);

   s1 = new TGEdit      (this, 101, 31);
   s2 = new TGStatic    (this, 102, 31);
   s3 = new TGCheckBox  (this, 103);
   s4 = new TGButton    (this, 104);
   s5 = new TGComboBox  (this, 105, 16);
   s6 = new TGEdit      (this, 106, 31);

   // set colors for some controls
   s1->SetColors (TColor(0,0,128));                   // Blue text
   s2->SetColors (TColor::LtYellow, TColor::Black);   // Yellow on black
   s5->SetColors (TColor::LtGreen, TColor::Gray);     // Green on dark gray
   s6->SetColors (TColor::White, TColor(0,128,128));  // White on cyan

   // create the grid
   grid = new TestGrid (this);

   // link controls to the grid.
   grid->Add (s1, "Edit 1");
   grid->Add (s5, "ComboBox 5");
   grid->Add (s6, "Edit 6", TGrid::Right);
   grid->Add (s2, "Static 2");
   grid->Add (s3, "CheckBox 3");
   grid->Add (s4, "Button 4");

   // uncomment next line to disable line swapping (ie CTRL+UP or DOWN
   // will move current line 1 line up or down
   // grid->SetLineSwapping (FALSE);  // line swapping is on by default
   }


// SetupWindow
void Test::SetupWindow ()
   {
   TGDialog::SetupWindow ();
   // stuffing data into TGComboBox
   int max = MAXLINE;
   if (max < 1000)
      max = 1000;
   for (int i = 0; i <= max; i++)
      {
      char string[5];
      sprintf (string, "%04d", i);
      s5->AddString (string);
      }
   }


// Respond to window resizing, passing the message to TGrid.
void Test::EvWindowPosChanged (WINDOWPOS far &wpos)
   {
   TGDialog::EvWindowPosChanged (wpos);
   // change other controls' position
   int h = HIWORD(GetDialogBaseUnits());
   int y = wpos.cy - 3*h;
   test->SetWindowPos (0, test->Attr.X, y, 0, 0, SWP_NOZORDER|SWP_NOSIZE);
   ok->SetWindowPos (0, ok->Attr.X, y, 0, 0, SWP_NOZORDER|SWP_NOSIZE);
   esc->SetWindowPos (0, esc->Attr.X, y, 0, 0, SWP_NOZORDER|SWP_NOSIZE);

   // change grid size
   grid->SetWindowPos (0, 0, 0, wpos.cx, y-2*h, SWP_NOZORDER);
   }


// test the RefreshItem and ChangeTitle methods here
void Test::TestRefresh ()
   {
   // let's change some column to test RefreshItem
   for (int i = 0; i < MAXLINE; i++)
      {
      int val = atoi (grid->data[i].str+8);
      val += i;
      sprintf (grid->data[i].str, "EDIT1 - %04d", val);

      grid->RefreshItem (i, 101);
      }

   // test ChangeTitle too
   const char *s = grid->GetTitle (101);
   int val = strlen(s) <= 9 ? 0 : atoi (s+9);
   char buffer[30];
   sprintf (buffer, "Edit 1 - %d", val+1);
   grid->ChangeTitle (101, buffer);
   }

/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// Class TestGrid                                                          //
//                                                                         //
//-------------------------------------------------------------------------//
//                                                                         //
// This class is derived from TGrid                                        //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

// constructor //////////////////////////////////////////////////////////////
TestGrid::TestGrid (TWindow *p)
        : TGrid (p, 100, TGrid::EnterNextColumn, TGrid::AutoNextLine,
                 TGrid::ReverseColor, -1, gAllAttrib)
   {
   // initialize some test data
   for (int i = 0; i < MAXLINE; i++)
      {
      char edstring[16];
      char edstring2[16];
      char ststring[16];
      char cbstring[5];
      sprintf (edstring,  "EDIT1 - %04d", i+1);
      sprintf (edstring2, "EDIT2 - %04d", MAXLINE-i);
      sprintf (ststring,  "STATIC - %04d", i+1);
      sprintf (cbstring,  "%04d", i+1);
      data[i].Init (edstring, edstring2, ststring, (i&1) == 1?TRUE:FALSE, cbstring);
      }
   // Change fonts: grid hass Arial, 10 points, bold.
   //               title has Times New Roman, 12 points, italic.
   int fontheight = MulDiv(TScreenDC().GetDeviceCaps (LOGPIXELSY),10,72);
   SetFont ("Arial", -fontheight, FW_BOLD, FALSE);
   fontheight = MulDiv(TScreenDC().GetDeviceCaps (LOGPIXELSY),12,72);
   SetTitleFont ("Times New Roman", -fontheight, FW_NORMAL, TRUE);
   }


// destructor ///////////////////////////////////////////////////////////////
TestGrid::~TestGrid ()
   {
   // dump data for verification
   FILE *fd = fopen ("dump.txt", "w");
   fprintf (fd, " %-30s ! %30s ! %-30s ! %-15s ! %s\n",
                "Edit 1", "Edit 2", "Static", "Combo", "Check");
   fprintf (fd, "--------------------------------+"
                "--------------------------------+"
                "--------------------------------+"
                "-----------------+-------\n");
   for (int i = 0; i < MAXLINE; i++)
      {
      fprintf (fd, " %-30s ! %30s ! %-30s ! %-15s ! %d\n",
            data[i].str,
            data[i].str2,
            data[i].stat,
            data[i].combo,
            data[i].chk);
      }
   fclose (fd);
   }


// CalcPos //////////////////////////////////////////////////////////////////
// change current line before we start editing the grid                    //
void TestGrid::CalcPos ()
   {
   TGrid::CalcPos ();
   Go (20);
   }


// AppendNewLine ////////////////////////////////////////////////////////////
// Overloaded TGrid::AppendNewLine; this is only necessary if:             //
//  1) maxLines == -1                                                      //
//  and                                                                    //
//  2) the maximum number of lines has to be controlled                    //
BOOL TestGrid::AppendNewLine (int &newline)
   {
   if (newline >= MAXLINE)
      return FALSE;
   else
      return TRUE;
#if 0
   BOOL result = newline < 30;
   return result;
#endif
   }


// TransferItem /////////////////////////////////////////////////////////////
// char *TransferItem : overloaded virtual function                        //
// this function returns the relevant data for line l and column id        //
// (data is transferred from memory to grid)                               //
char *TestGrid::TransferItem (int line, int id)
   {
#if 0
   static char buffer[100];
   char *result = 0;
   if (line < 30)
      {
      wsprintf(buffer, "%d %d", line, id);
      result = buffer;
      }
   return result;
#endif
   if (line >= MAXLINE)
      return 0;

   switch (id)
      {
      case 101:
         return data[line].str;
      case 102:
         return data[line].stat;
      case 103:
         return data[line].chk ? "1" : "0";
      case 104:
         return "Button";
      case 105:
         return data[line].combo;
      case 106:
         return data[line].str2;
      }

   return 0;
   }


// TransferItem /////////////////////////////////////////////////////////////
// void  TransferItem : overloaded virtual function                        //
// this function retrieves data from the grid and stores it in memory      //
// (data is transferred from grid to memory)                               //
void TestGrid::TransferItem (int line, int id, const char *string)
   {
   if (line >= MAXLINE || string == 0)
      return;

   switch (id)
      {
      case 101:
         strcpy (data[line].str, string);
         break;
      case 102:
         strcpy (data[line].stat, string);
         break;
      case 103:
         data[line].chk = (*string != '0' ? TRUE : FALSE);
         break;
      case 105:
         strcpy (data[line].combo, string);
         break;
      case 106:
         strcpy (data[line].str2, string);
         break;
      }
   }


// InsertLine ///////////////////////////////////////////////////////////////
// BOOL InsertLine : overloaded virtual function                           //
// We have to move data on our behalf, then clear the current line         //
// return TRUE to allow tell TGrid line insert is allowed                  //
BOOL TestGrid::InsertLine (int line)
   {
   movmem (data+line, data+line+1, sizeof(Data)*(MAXLINE-line-1));
   memset (data+line, 0, sizeof(Data));

   char string[16];
   sprintf (string, "INSERTED - %04d", line+1);
   data[line].Init (string, string, string, TRUE, "0000");

   return TRUE;
   }


// DeleteLine ///////////////////////////////////////////////////////////////
// BOOL InsertLine : overloaded virtual function                           //
// We have to move data on our behalf, then clear the last line            //
// return TRUE to allow tell TGrid line delete is allowed                  //
BOOL TestGrid::DeleteLine (int line)
   {
   movmem (data+line+1, data+line, sizeof(Data)*(MAXLINE-line-1));
   memset (data+MAXLINE-1, 0, sizeof(Data));

   char string[16];
   sprintf (string, "DELETED - %04d", line+1);
   data[MAXLINE-1].Init (string, string, string, FALSE, "1000");

   return TRUE;
   }

// EOF

